@@ -87,6 +87,13 @@ TurboLucene_ uses the following configuration options:
     allows for all stopword files to be customized without needing to specify
     full paths for every one.  Normally you should not need to override this.
     (Default: the ``stopwords`` directory in the `turbolucene` package)
+  **turbolucene.force_lock_release**:
+    If this is set to True, then if TurboLucne has troubles opening an index,
+    it will try to force the release of any write lock that may exist and try
+    again.  The write lock is to prevent multiple processes writing to the
+    same index at the same time, but if the TurboLucne-based project is killed,
+    the lock gets left behind.  This setting let you override the default
+    behaviour.  (Default: ``True`` in development and ``False`` in production)
 
 All fields are optional, but at the minimum, you will likely want to specify
 ``turbolucene.search_fields``.
@@ -115,9 +122,9 @@ __author__ = 'Krys Wilken'
 __contact__ = 'krys AT krys DOT ca'
 __copyright__ = '(c) 2007 Krys Wilken'
 __license__ = 'MIT'
-__version__ = '0.2'
+__version__ = '0.2.1'
 __api_version__ = '2.0'
-__revision__ = '$Id: __init__.py 47 2007-04-01 22:36:05Z krys $'
+__revision__ = '$Id: __init__.py 60 2007-05-02 03:29:02Z krys $'
 __docformat__ = 'restructuredtext en'
 __all__ = ['start', 'add', 'update', 'remove', 'search', 'Document', 'Field',
   'STORE', 'COMPRESS', 'TOKENIZED', 'UN_TOKENIZED']
@@ -130,7 +137,7 @@ from Queue import Queue
 from os.path import exists, join, isabs
 from logging import getLogger
 from atexit import register
-from codecs import EncodedFile, open as codecs_open
+import codecs
 
 #---  Framework imports
 from turbogears import scheduler, config
@@ -141,7 +148,7 @@ from pkg_resources import resource_stream # pylint: disable-msg=E0611
 #---  Third-party imports
 import PyLucene
 from PyLucene import (PythonThread, IndexModifier, JavaError, Term,
-  IndexSearcher, MultiFieldQueryParser)
+  IndexSearcher, MultiFieldQueryParser, FSDirectory)
 # For use in make_document
 from PyLucene import Document, Field
 
@@ -327,11 +334,11 @@ def _read_stopwords(file_path, encoding):
         if not isabs(file_path):
             file_path = join(stopwords_base_path, file_path)
         _log.info(u'Reading custom stopwords file "%s"' % file_path)
-        stopwords_file = codecs_open(file_path, 'r', encoding)
+        stopwords_file = codecs.open(file_path, 'r', encoding)
     else:
         _log.info(u'Reading default stopwords file "%s"' % file_path)
-        stopwords_file = EncodedFile(resource_stream(__name__, join(
-          u'stopwords', file_path)), encoding)
+        stopwords_file = codecs.getreader(encoding)(resource_stream(__name__,
+          join(u'stopwords', file_path)))
     stopwords = []
     for line in stopwords_file:
         # Stopword files can have comments after a '|' character on each line.
@@ -634,8 +641,9 @@ class _Indexer(PythonThread):
         One PyLucene index is created/opened for each of the configured
         supported languages.
 
-        This method uses the ``turbolucene.default_language`` and
-        ``turbolucene.languages`` configuration settings.
+        This method uses the ``turbolucene.default_language``,
+        ``turbolucene.languages`` and ``turbolucene.force_lock_release``
+        configuration settings.
 
         :Parameters:
           make_document : callable
@@ -660,13 +668,30 @@ class _Indexer(PythonThread):
         self._indexes = {}
         default_language = config.get('turbolucene.default_language',
           _DEFAULT_LANGUAGE)
+        default_force_lock_release = config.get('server.environment',
+          'development').lower() == 'development' and True or False
+        force_lock_release = config.get('turbolucene.force_lock_release',
+          default_force_lock_release)
         # Create indexes
         languages = config.get('turbolucene.languages', [default_language])
         for language in languages:
             index_path = _get_index_path(language)
-            self._indexes[language] = IndexModifier(index_path,
-              _analyzer_factory(language), not exists(index_path) and True or
-              False)
+            analyzer = _analyzer_factory(language)
+            create_path = not exists(index_path) and True or False
+            try:
+                self._indexes[language] = IndexModifier(index_path, analyzer,
+                  create_path)
+            except JavaError, error:
+                if not error.getJavaException().getClass().getName(
+                  ) == 'java.io.IOException' or not force_lock_release:
+                    raise
+                _log.warn('Error opening index "%s".  '
+                  'turbolucene.force_lock_release is True, trying to force '
+                  'lock release.' % index_path)
+                FSDirectory.getDirectory(index_path, False).makeLock(
+                  'write.lock').release()
+                self._indexes[language] = IndexModifier(index_path, analyzer,
+                  create_path)
         self.start()
 
     def __call__(self, task, object_=None, language=None):
